% California Oil Spill study - full dataset

clear all
clc

global B_backup; % this one is for storing B in case iterations are interrupted with ctrl-c


%% ****************************  loading data  ****************************


EstimOpt.DataFile = ('cos_data_1.mat');
DATA = load(EstimOpt.DataFile);


%% ****************************  model specification  ****************************


DATA.Y = DATA.Y;

DATA.X = DATA.X;
DATA.X(:,end) = -DATA.X(:,end)/100;
EstimOpt.NamesA = {'Status quo';'-Cost'};



%% ****************************  specifying input ****************************

DATA.filter = ones(size(DATA.Y))==1;
% DATA.filter(201:end) = 0; % for Table 3 results

INPUT.Y = DATA.Y(DATA.filter);
INPUT.Xa = DATA.X(DATA.filter,:);


%% ****************************  sample characteristics ****************************


EstimOpt.NCT = 1; % Number of choice tasks per person
EstimOpt.NAlt = 2; % Number of alternatives
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt; % numel(unique(DATA.nr))


%% **************************** estimation and optimization options ****************************


EstimOpt.eps = 1.e-12; % overall precision level

[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NumGrad = 1; % 1
OptimOpt.GradObj = 'off'; % 'off'
OptimOpt.FinDiffType = 'central    '; % 'forward'


%% **************************** MNL ****************************

Results.MNL1 = MNL(INPUT,Results,EstimOpt,OptimOpt);

EstimOpt.ExpB = [0,1];
Results.MNL2 = MNL(INPUT,Results,EstimOpt,OptimOpt);

EstimOpt.ExpB = [1,1];
Results.MNL3 = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% Format results table


sim1 = 1e8;
z0 = norminv(0.975,0,1);
RES = zeros(18,3);


RES([1,3],1) = Results.MNL1.bhat;
RES([1,3],2) = Results.MNL2.bhat;
RES([1,3],3) = Results.MNL3.bhat;
RES([2,4],1) = Results.MNL1.std;
RES([2,4],2) = Results.MNL2.std;
RES([2,4],3) = Results.MNL3.std;
RES(5,1) = RES(1,1) ./ RES(3,1) * 100;
RES(5,2) = RES(1,2) ./ exp(RES(3,2)) * 100;
RES(5,3) = exp(RES(1,3)) ./ exp(RES(3,3)) * 100;

B0 = Results.MNL1.bhat;
V0 = Results.MNL1.ihess;
rng(15485863);
Bsim0 = mvnrnd(B0,V0,sim1);
BC0 = Bsim0(:,1)./Bsim0(:,2)*100;
RES(6,1) = median(BC0);
RES(7,1) = mean(BC0);
RES(8,1) = RatioDeltaSE(Results.MNL1.bhat,Results.MNL1.ihess,0)*100; % s.e. - delta 
RES(9,1) = std(BC0); % s.e. - KR
RES(10,1) = RES(5,1) - z0*RES(8,1); % 95% c.i. - delta
RES(11,1) = RES(5,1) + z0*RES(8,1); % 95% c.i. - delta
RES(12,1) = quantile(BC0,0.025); % 95% c.i. - KR
RES(13,1) = quantile(BC0,0.975); % 95% c.i. - KR
[a1,a2,a3,a4] = RatioFiellerBounds(RES(1,1),RES(3,1),RES(2,1),RES(4,1),Results.MNL1.ihess(2,1)./RES(2,1)./RES(4,1));
RES(14,1) = a2*100; % 95% c.i. - Fieller
RES(15,1) = a3*100; % 95% c.i. - Fieller
RES(16,1) = Results.MNL1.LL;
RES(17,1) = Results.MNL1.stats(5,1);
RES(18,1) = Results.MNL1.stats(7,1);

BC0(BC0 < -170) = [];
BC0(BC0 > 270) = [];

set(gca,'Position',[.03 .07 .94 .92]);
set(gcf, 'Color', 'w')
set(gcf,'defaultlinelinewidth',2);
set(gcf,'DefaultAxesColorOrder',[0 0 1;1 0 0;0 1 0]);
set(gcf,'Units','normal');
ksdensity(BC0);
% set(gca,'XLim',[WTP(5,1) WTP(6,1)]);
% set(gca,'YLim',[0.002/5 0.012]);
hold all


B0 = Results.MNL2.bhat;
V0 = Results.MNL2.ihess;
rng(15485863);
Bsim0 = mvnrnd(B0,V0,sim1);
BC0 = Bsim0(:,1)./exp(Bsim0(:,2))*100;
RES(6,2) = median(BC0);
RES(7,2) = mean(BC0);
RES(8,2) = RatioDeltaSE(Results.MNL2.bhat,Results.MNL2.ihess,1)*100; % s.e. - delta 
RES(9,2) = std(BC0); % s.e. - KR
RES(10,2) = RES(5,2) - z0*RES(8,2); % 95% c.i. - delta
RES(11,2) = RES(5,2) + z0*RES(8,2); % 95% c.i. - delta
RES(12,2) = quantile(BC0,0.025); % 95% c.i. - KR
RES(13,2) = quantile(BC0,0.975); % 95% c.i. - KR
[a1,a2,a3,a4] = RatioFiellerBounds(RES(1,2),RES(3,2),RES(2,2),RES(4,2),Results.MNL2.ihess(2,1)./RES(2,2)./RES(4,2),1);
RES(14,2) = a2*100; % 95% c.i. - Fieller
RES(15,2) = a3*100; % 95% c.i. - Fieller
RES(16,2) = Results.MNL2.LL;
RES(17,2) = Results.MNL2.stats(5,1);
RES(18,2) = Results.MNL2.stats(7,1);

BC0(BC0 < -170) = [];
BC0(BC0 > 270) = [];
ksdensity(BC0);

B0 = Results.MNL3.bhat;
V0 = Results.MNL3.ihess;
rng(15485863);
Bsim0 = mvnrnd(B0,V0,sim1);
BC0 = exp(Bsim0(:,1))./exp(Bsim0(:,2))*100;
RES(6,3) = median(BC0);
RES(7,3) = mean(BC0);
RES(8,3) = RatioDeltaSE(Results.MNL3.bhat,Results.MNL3.ihess,2)*100; % s.e. - delta 
RES(9,3) = std(BC0); % s.e. - KR
RES(10,3) = RES(5,3) - z0*RES(8,3); % 95% c.i. - delta
RES(11,3) = RES(5,3) + z0*RES(8,3); % 95% c.i. - delta
RES(12,3) = quantile(BC0,0.025); % 95% c.i. - KR
RES(13,3) = quantile(BC0,0.975); % 95% c.i. - KR
[a1,a2,a3,a4] = RatioFiellerBounds(RES(1,3),RES(3,3),RES(2,3),RES(4,3),Results.MNL3.ihess(2,1)./RES(2,3)./RES(4,3),2);
RES(14,3) = a2*100; % 95% c.i. - Fieller
RES(15,3) = a3*100; % 95% c.i. - Fieller
RES(16,3) = Results.MNL3.LL;
RES(17,3) = Results.MNL3.stats(5,1);
RES(18,3) = Results.MNL3.stats(7,1);

BC0(BC0 < -170) = [];
BC0(BC0 > 270) = [];
ksdensity(BC0);
hold off

l1 = legend( 'linear spec. for C', 'exponential spec. for C', 'exponential spec. for C and ASC');
set(l1, 'Position', [.665,.85,.15,.01]);
set(gca,'YTickLabelMode','Manual')
set(gca, 'YTick', [])
set(gca,'XLim',[50 114])
saveas(gcf,'F11','png')

% set(gca,'XLim',[-150 250])
% saveas(gcf,'F12','png')