% Alternative Fuels Vehicle DCE Study 

clear all
clc

global B_backup; % this one is for storing B in case iterations are interrupted with ctrl-c


%% ****************************  loading data  ****************************


EstimOpt.DataFile = ('afv_data_1.mat');
DATA = load(EstimOpt.DataFile);


%% ****************************  model specification  ****************************


DATA.Y = DATA.Y;

DATA.X = [DATA.range, DATA.electric, DATA.hybrid, DATA.medium_or_high_performance, DATA.high_performance, DATA.price, DATA.operating_cost];
EstimOpt.NamesA = {'range', 'electric','hybrid','p_medium','p_high','c_purchase','c_operate'};


%% ****************************  specifying input ****************************


DATA.filter = ones(size(DATA.Y))==1;

INPUT.Y = DATA.Y(DATA.filter);
INPUT.Xa = DATA.X(DATA.filter,:);


%% ****************************  sample characteristics ****************************


EstimOpt.NCT = 15; 
EstimOpt.NAlt = 3; 
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt; 


%% **************************** estimation and optimization options ****************************


EstimOpt.eps = 1.e-12; % overall precision level

[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NRep = 1e4;
EstimOpt.NumGrad = 1; % 1
OptimOpt.GradObj = 'on'; % 'off'
OptimOpt.FinDiffType = 'central'; % 'forward'


%% **************************** MNL ****************************


Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% **************************** MXL ****************************

EstimOpt.Dist = [1,1,0,0,0,0,-1,-1];

B_backup = [-0.732807742117448;-1.79078615749893;0.439529881921599;0.530979749786149;0.0770045261497994;0.474823170940744;0.0135607307780839;0.591511660367994;1.26584577595331;0.974525021506331;0.569599786616851;0.351399028473577;0;0];
Results.MXL1 = MXL(INPUT,Results,EstimOpt,OptimOpt);

EstimOpt.ExpB = zeros(14,1);
EstimOpt.ExpB(6:7) = 1;
B_backup(6:7) = log(B_backup(6:7));
Results.MXL2 = MXL(INPUT,Results,EstimOpt,OptimOpt);


%% Format results table

sim1 = 1e8;
z0 = norminv(0.975,0,1);

RES = NaN(28,4);
RES([1,3,5,7,9,11,13],1) = Results.MXL1.bhat(1:7);
RES([1,3,5,7,9,11,13],3) = Results.MXL2.bhat(1:7);
RES([1,3,5,7,9,11,13],2) = Results.MXL1.bhat(8:14);
RES([1,3,5,7,9,11,13],4) = Results.MXL2.bhat(8:14);
RES([2,4,6,8,10,12,14],1) = Results.MXL1.std(1:7);
RES([2,4,6,8,10,12,14],3) = Results.MXL2.std(1:7);
RES([2,4,6,8,10,12,14],2) = Results.MXL1.std(8:14);
RES([2,4,6,8,10,12,14],4) = Results.MXL2.std(8:14);

RES(15,1) = exp(RES(1,1)+0.5*RES(1,2)^2)./RES(13,1); % ratio of coefficients
RES(15,3) = exp(RES(1,3)+0.5*RES(1,4)^2)./exp(RES(13,3));

B0 = Results.MXL1.bhat([1,8,7]);
V0 = Results.MXL1.ihess([1,8,7],[1,8,7]);
rng(15485863);
Bsim0 = mvnrnd(B0,V0,sim1);
% for i = 1:sim1
%     B0_i(i) = normrnd(Bsim0(i,1),Bsim0(i,8),[sim2,1]);
% end
WTP_i = [(exp(Bsim0(:,1)))./Bsim0(:,3) , (exp(Bsim0(:,1) + 0.5*Bsim0(:,2).^2))./Bsim0(:,3)];
RES(16,1) = mean(WTP_i(:,1)); % median KR
RES(17,1) = mean(WTP_i(:,2)); % mean KR
RES(18,1) = RatioDeltaSE(B0,V0,3); % s.e. delta
RES(19,1) = std(WTP_i(:,2)); % s.e. KR
RES(20,1) = RES(15,1) - z0*RES(18,1); % 95% c.i. delta
RES(21,1) = RES(15,1) + z0*RES(18,1);
RES(22,1) = quantile(WTP_i(:,2),0.025); % 95% c.i. KR
RES(23,1) = quantile(WTP_i(:,2),0.975);
[a1,a2,a3,a4] = RatioFiellerBounds(B0,V0,3);
RES(24,1) = a2; % 95% c.s. Fieller
RES(25,1) = a3;
RES(26,1) = Results.MXL1.LL;
RES(27,1) = Results.MXL1.stats(5);
RES(28,1) = Results.MXL1.stats(7);

set(gca,'Position',[.03 .07 .94 .92]);
set(gcf, 'Color', 'w')
set(gcf,'defaultlinelinewidth',2);
set(gcf,'DefaultAxesColorOrder',[0 0 1;1 0 0;0 1 0]);
set(gcf,'Units','normal');
WTP_i(WTP_i(:,2) < 0,:) = [];
WTP_i(WTP_i(:,2) > 150,:) = [];
ksdensity(WTP_i(:,2));
% set(gca,'XLim',[WTP(5,1) WTP(6,1)]);
% set(gca,'YLim',[0.002/5 0.012]);
hold all

B0 = Results.MXL2.bhat([1,8,7]);
V0 = Results.MXL2.ihess([1,8,7],[1,8,7]);
rng(15485863);
Bsim0 = mvnrnd(B0,V0,sim1);
% for i = 1:sim1
%     B0_i(i) = normrnd(Bsim0(i,1),Bsim0(i,8),[sim2,1]);
% end
WTP_i = [(exp(Bsim0(:,1)))./exp(Bsim0(:,3)) , (exp(Bsim0(:,1) + 0.5*Bsim0(:,2).^2))./exp(Bsim0(:,3))];
RES(16,3) = mean(WTP_i(:,1)); % median KR
RES(17,3) = mean(WTP_i(:,2)); % mean KR
RES(18,3) = RatioDeltaSE(B0,V0,4); % s.e. delta
RES(19,3) = std(WTP_i(:,2)); % s.e. KR
RES(20,3) = RES(15,3) - z0*RES(18,3); % 95% c.i. delta
RES(21,3) = RES(15,3) + z0*RES(18,3);
RES(22,3) = quantile(WTP_i(:,2),0.025); % 95% c.i. KR
RES(23,3) = quantile(WTP_i(:,2),0.975);
[a1,a2,a3,a4] = RatioFiellerBounds(B0,V0,4);
RES(24,3) = a2; % 95% c.s. Fieller
RES(25,3) = a3;
RES(26,3) = Results.MXL2.LL;
RES(27,3) = Results.MXL2.stats(5);
RES(28,3) = Results.MXL2.stats(7);

WTP_i(WTP_i(:,2) < 0,:) = [];
WTP_i(WTP_i(:,2) > 150,:) = [];
ksdensity(WTP_i(:,2));
hold off

set(gca,'XLim',[0 120]);
set(gca,'YTickLabelMode','Manual')
set(gca, 'YTick', [])

l1 = legend( 'linear spec. for C', 'exponential spec. for C');
set(l1, 'Position', [.665,.85,.15,.01]);
saveas(gcf,'F2','png')